@@ -115,7 +115,7 @@ from typing import TYPE_CHECKING, get_type_hints
 from typing import AnyStr, Deque, Optional, Tuple, Union, Mapping, Any, List, TypeVar, Iterable, ByteString, Callable
 
 from refinery.lib.frame import Chunk
-from refinery.lib.tools import isbuffer, infinitize, one
+from refinery.lib.tools import isbuffer, infinitize, one, normalize_to_identifier
 from refinery.lib.meta import is_valid_variable_name, metavars
 
 if TYPE_CHECKING:
@@ -381,7 +381,7 @@ class DelayedArgumentDispatch:
         self.units = {}
 
     def _get_unit(self, name: str, *args) -> Unit:
-        name = name.replace('-', '_')
+        name = normalize_to_identifier(name)
         uhash = hash((name,) + args)
         if uhash in self.units:
             return self.units[uhash]
@@ -886,21 +886,26 @@ class DelayedArgument(LazyEvaluation):
         return LazyPythonExpression(expression)
 
     @handler.register('btoi')
-    def btoi(self, binary: ByteString, size=None) -> Iterable[int]:
+    def btoi(self, binary: ByteString, size=None, step=None) -> Iterable[int]:
         """
-        The modifier `btoi[size=0]:data` uses `refinery.lib.chunks.unpack` to convert a sequence
-        of bytes into a sequence of integers by unpacking them. The `expression` parameter is parsed
-        with `refinery.lib.argformats.multibin` yielding this byte string. The optional parameter
-        `size` has to be an integer expression whose absolute value gives the size of each encoded
-        number in bytes. Its default value is `0`, which corresponds to choosing the size
-        automatically in the following manner: If the length of the buffer is uneven, the value `1`
-        is chosen. If the length modulo `4` is nonzero, the value `2` is chosen. If the length is
-        divisible by `4`, then `4` is chosen. To unpack as big endian as opposed to the default
-        little endian, a negative `size` has to be specified. The absolute value of `size` will be
-        used.
+        The modifier `btoi[size=0,step=0]:data` uses `refinery.lib.chunks.unpack` to convert a
+        sequence of bytes into a sequence of integers.
+
+        The optional parameter `size` has to be an integer expression whose absolute value gives
+        the size of each encoded number in bytes. Its default value is `0`, which corresponds to
+        choosing the size automatically in the following manner: If the length of the buffer is
+        uneven, the value 1 is chosen. If the length modulo 4 is nonzero, the value 2 is chosen. If
+        the length is divisible by 4, then 4 is chosen. To unpack as big endian as opposed to the
+        default little endian, a negative value for `size` has to be specified. The absolute value
+        of `size` will be used.
+
+        By default, integers are parsed from the input buffer at offsets that are integer multiples
+        of the block size. The optional parameter `step` can be used to override this behavior. For
+        example, `btoi[2,1]` can be used to read 16-bit values at each byte offset.
         """
         from refinery.lib import chunks
         size = int(size, 0) if size else 0
+        step = int(step, 0) if step else 0
         bigE = size < 0
         size = abs(size)
         if not size:
@@ -911,7 +916,7 @@ class DelayedArgument(LazyEvaluation):
                 size = 2
             else:
                 size = 4
-        return list(chunks.unpack(binary, size, bigE))
+        return list(chunks.unpack(binary, size, bigE, step))
 
     @handler.register('itob')
     def itob(self, integers: Iterable[int], size=None) -> ByteString:
@@ -1069,7 +1074,7 @@ class DelayedArgument(LazyEvaluation):
                 return update(meta, A=A)
             meta = dict(metavars(data))
             A = seed and seed(meta) or 0
-            F = feed and feed(meta, A=A) or A
+            F = feed(meta, A=A) if feed else A
             S = skip
             while True:
                 if not S:
@@ -1079,7 +1084,7 @@ class DelayedArgument(LazyEvaluation):
                 A = accumulate(A)
                 if mask:
                     A &= mask
-                F = feed and feed(meta, A=A) or A
+                F = feed(meta, A=A) if feed else A
         try:
             update(A=seed())
         except ParserVariableMissing:
@@ -1124,8 +1129,11 @@ class DelayedArgument(LazyEvaluation):
         reduction = PythonExpression(reduction, all_variables_allowed=True)
 
         def finalize(data: Optional[Chunk] = None):
+            def _reduction(S, B):
+                v = reduction(args, S=S, B=B)
+                return v
             args = dict(metavars(data))
-            return reduce(lambda S, B: reduction(args, S=S, B=B), it, seed and seed(args) or 0)
+            return reduce(_reduction, it, seed and seed(args) or 0)
 
         try:
             return finalize()
@@ -1265,12 +1273,13 @@ class DelayedRegexpArgument(DelayedArgument):
 
         return expression.encode('latin-1')
 
-    @handler.register('yara')
+    @handler.register('yara', 'Y')
     def yara(self, pattern: bytes) -> bytes:
         """
-        The handler `yara:pattern` converts YARA syntax wildcard hexadecimal expressions
-        into regular expressions. For example, `D?` is translated to `[\\xD0-\\xDF]`, the
-        expression `[2-6]` becomes `.{2,6}`, and `?D` becomes the following substring:
+        The handler `yara:pattern` or `Y:pattern` converts YARA syntax wildcard hexadecimal
+        expressions into standard regular expressions. For example, the string `D?` is
+        translated to `[\\xD0-\\xDF]`, the expression `[2-6]` becomes `.{2,6}`, and `?D`
+        becomes the following substring:
         ```
         [\\x0D\\x1D\\x2D\\x3D\\x4D\\x5D\\x6D\\x7D\\x8D\\x9D\\xAD\\xBD\\xCD\\xDD\\xED\\xFD]
         ```
