@@ -10,11 +10,14 @@ The provided interface is the same for all executables. It powers the following
 """
 from __future__ import annotations
 
+import sys
+
 from typing import TYPE_CHECKING, NamedTuple
 from os import devnull as DEVNULL
 from abc import ABC, abstractmethod
 from enum import Enum
 from functools import lru_cache
+from uuid import uuid4
 
 from macholib.MachO import load_command, MachO, MachOHeader
 from pefile import PE as PEFile, SectionStructure, MACHINE_TYPE, DIRECTORY_ENTRY
@@ -146,26 +149,23 @@ class ArchItem(NamedTuple):
     id: int
     pointer_size: int
 
-
-def _arch_item(pointer_size):
-    counter = getattr(_arch_item, 'counter', 0)
-    result = ArchItem(counter, pointer_size)
-    _arch_item.counter = counter + 1
-    return result
+    @classmethod
+    def New(cls, pointer_size: int):
+        return cls(uuid4(), pointer_size)
 
 
 class Arch(ArchItem, Enum):
-    X8632 = _arch_item(32)
-    X8664 = _arch_item(64)
-    ARM32 = _arch_item(32)
-    ARM64 = _arch_item(64)
-    MIPS16 = _arch_item(16)
-    MIPS32 = _arch_item(32)
-    MIPS64 = _arch_item(64)
-    PPC32 = _arch_item(32)
-    PPC64 = _arch_item(64)
-    SPARC32 = _arch_item(32)
-    SPARC64 = _arch_item(64)
+    X32 = ArchItem.New(32)
+    X64 = ArchItem.New(64)
+    ARM32 = ArchItem.New(32)
+    ARM64 = ArchItem.New(64)
+    MIPS16 = ArchItem.New(16)
+    MIPS32 = ArchItem.New(32)
+    MIPS64 = ArchItem.New(64)
+    PPC32 = ArchItem.New(32)
+    PPC64 = ArchItem.New(64)
+    SPARC32 = ArchItem.New(32)
+    SPARC64 = ArchItem.New(64)
 
 
 class LT(str, Enum):
@@ -177,6 +177,7 @@ class ET(str, Enum):
     ELF = 'ELF'
     MachO = 'MachO'
     PE = 'PE'
+    BLOB = 'BLOB'
 
 
 class BO(str, Enum):
@@ -307,7 +308,10 @@ class Executable(ABC):
 
     @property
     def data(self) -> memoryview:
-        return memoryview(self._data).toreadonly()
+        view = memoryview(self._data)
+        if sys.version_info >= (3, 8):
+            view = view.toreadonly()
+        return view
 
     @property
     def pointer_size(self) -> int:
@@ -376,6 +380,36 @@ class Executable(ABC):
         ...
 
 
+class ExecutableCodeBlob(Executable):
+
+    _head: Type[None] = None
+    _type = ET.BLOB
+    _byte_order: BO
+    _arch: Arch
+
+    def __init__(self, data, base=None, arch: Arch = Arch.X32, byte_order: BO = BO.LE):
+        super().__init__(None, data, base)
+        self._byte_order = byte_order
+        self._arch = arch
+
+    def image_defined_base(self) -> int:
+        return 0
+
+    def byte_order(self) -> BO:
+        return self._byte_order
+
+    def arch(self) -> Arch:
+        return self._arch
+
+    def sections(self) -> Generator[Section, None, None]:
+        r = Range(0, len(self.data))
+        yield Section('blob', r, r)
+
+    def segments(self, populate_sections=False) -> Generator[Segment, None, None]:
+        for s in self.sections():
+            yield s.as_segment(populate_sections=populate_sections)
+
+
 class ExecutablePE(Executable):
 
     _head: PEFile
@@ -423,7 +457,9 @@ class ExecutablePE(Executable):
     def sections(self) -> Generator[Section, None, None]:
         sections: Iterable[SectionStructure] = iter(self._head.sections)
         ib = self.image_defined_base()
+        exeblob = True
         for section in sections:
+            exeblob = False
             p_lower = section.PointerToRawData
             p_upper = p_lower + section.SizeOfRawData
             v_lower = section.VirtualAddress + ib
@@ -434,6 +470,11 @@ class ExecutablePE(Executable):
                 Range(p_lower, p_upper),
                 Range(v_lower, v_upper),
             )
+        if exeblob:
+            size = len(self._data)
+            v_lower = self.image_defined_base()
+            v_upper = v_lower + size
+            yield Section('.CODE', Range(0, size), Range(v_lower, v_upper))
 
     def segments(self, populate_sections=False) -> Generator[Segment, None, None]:
         for section in self.sections():
@@ -444,8 +485,8 @@ class ExecutablePE(Executable):
         arch = MACHINE_TYPE[arch]
         try:
             return {
-                'IMAGE_FILE_MACHINE_I386'   : Arch.X8632,
-                'IMAGE_FILE_MACHINE_AMD64'  : Arch.X8664,
+                'IMAGE_FILE_MACHINE_I386'   : Arch.X32,
+                'IMAGE_FILE_MACHINE_AMD64'  : Arch.X64,
                 'IMAGE_FILE_MACHINE_ARM'    : Arch.ARM32,
                 'IMAGE_FILE_MACHINE_THUMB'  : Arch.ARM32,
                 'IMAGE_FILE_MACHINE_ARMNT'  : Arch.ARM64,
@@ -521,8 +562,8 @@ class ExecutableELF(Executable):
             return {
                 'EM_SPARC'   : Arch.SPARC32,
                 'EM_SPARCV9' : Arch.SPARC64,
-                'EM_386'     : Arch.X8632,
-                'EM_X86_64'  : Arch.X8664,
+                'EM_386'     : Arch.X32,
+                'EM_X86_64'  : Arch.X64,
                 'EM_MIPS'    : Arch.MIPS32,
                 'EM_PPC'     : Arch.PPC32,
                 'EM_PPC64'   : Arch.PPC64,
@@ -603,8 +644,8 @@ class ExecutableMachO(Executable):
             arch = F'UNKNOWN(0x{cputype:X})'
         try:
             return {
-                'X86'       : Arch.X8632,
-                'X86_64'    : Arch.X8664,
+                'X86'       : Arch.X32,
+                'X86_64'    : Arch.X64,
                 'ARM'       : Arch.ARM32,
                 'SPARC'     : Arch.SPARC32,
                 'POWERPC'   : Arch.PPC32,
