@@ -12,6 +12,7 @@ import os
 import sys
 import io
 import warnings
+import re
 
 from typing import ByteString, Callable, Generator, Iterable, Optional, Tuple, TypeVar
 from math import log
@@ -65,7 +66,7 @@ def get_terminal_size(default=0):
     return default if width < 2 else width - 1
 
 
-def terminalfit(text: str, delta: int = 0, width: int = 0, **kw) -> str:
+def terminalfit(text: str, delta: int = 0, width: int = 0, parsep: str = '\n\n', **kw) -> str:
     """
     Reformats text to fit the given width while not mangling bullet point lists.
     """
@@ -109,7 +110,7 @@ def terminalfit(text: str, delta: int = 0, width: int = 0, **kw) -> str:
                     continue
             yield '\n'.join(textwrap.wrap(p, width, **kw))
 
-    return '\n\n'.join(fitted(text.split('\n\n')))
+    return parsep.join(fitted(text.split('\n\n')))
 
 
 def documentation(unit):
@@ -362,3 +363,54 @@ def isodate(iso: str) -> Optional[datetime.datetime]:
             return datetime.datetime.strptime(iso, "%Y-%m-%dT%H:%M:%S")
     except ValueError:
         return None
+
+
+def integers_of_slice(s: slice) -> Iterable[int]:
+    """
+    Returns an iterable that iterates the integers in the range given by the input slice.
+    """
+    if s.stop is None:
+        return itertools.count(s.start or 0, s.step or 1)
+    else:
+        return range(s.start or 0, s.stop, s.step or 1)
+
+
+def normalize_word_separators(words: str, unified_separator: str, strip: bool = True):
+    """
+    For a sequence of words separated by whitespace, punctuation, slashes, dashes or underscores,
+    normalize all occurrences of one or more of these seprators to one given symbol. Leading and
+    trailing occurrences of separators are removed.
+    """
+    normalized = re.sub('[-\\s_.,;:/\\\\]+', unified_separator, words)
+    if strip:
+        normalized = normalized.strip(unified_separator)
+    return normalized
+
+
+def normalize_to_display(words: str, strip: bool = True):
+    """
+    Normalizes all separators to dashes.
+    """
+    return normalize_word_separators(words, '-', strip)
+
+
+def normalize_to_identifier(words: str, strip: bool = True):
+    """
+    Normalizes all separators to underscores.
+    """
+    return normalize_word_separators(words, '_', strip)
+
+
+def typename(thing):
+    """
+    Determines the name of the type of an object.
+    """
+    if not isinstance(thing, type):
+        thing = type(thing)
+    mro = [c for c in thing.__mro__ if c is not object]
+    if mro:
+        thing = mro[~0]
+    try:
+        return thing.__name__
+    except AttributeError:
+        return repr(thing)
