@@ -194,14 +194,36 @@ from argparse import (
     ZERO_OR_MORE
 )
 
-from refinery.lib.argformats import pending, manifest, multibin, numseq, number, sliceobj, VariableMissing, ParserVariableMissing
 from refinery.lib.argparser import ArgumentParserWithKeywordHooks, ArgparseError
-from refinery.lib.tools import documentation, isstream, lookahead, autoinvoke, one, skipfirst, isbuffer
 from refinery.lib.frame import Framed, Chunk
 from refinery.lib.structures import MemoryFile
 from refinery.lib.environment import LogLevel, Logger, environment, logger
 from refinery.lib.types import ByteStr, Singleton
 
+from refinery.lib.argformats import (
+    manifest,
+    multibin,
+    number,
+    numseq,
+    ParserVariableMissing,
+    pending,
+    regexp,
+    sliceobj,
+    VariableMissing,
+)
+
+from refinery.lib.tools import (
+    autoinvoke,
+    documentation,
+    isbuffer,
+    isstream,
+    lookahead,
+    normalize_to_display,
+    normalize_to_identifier,
+    one,
+    skipfirst,
+)
+
 
 ByteIO = MemoryFile[ByteStr]
 
@@ -312,6 +334,8 @@ class Arg(Argument):
     class delete: pass
     class omit: pass
 
+    args: List[str]
+
     def __init__(
         self, *args: str,
             action   : Union[omit, str]           = omit, # noqa
@@ -377,14 +401,14 @@ class Arg(Argument):
         if isinstance(value, str):
             try: return cls[value]
             except KeyError: pass
-            needle = value.upper().replace('-', '_')
-            for item in cls:
-                if item.name.upper() == needle:
-                    return item
+            needle = normalize_to_identifier(value).casefold()
+            for item in cls.__members__:
+                if item.casefold() == needle:
+                    return cls[item]
         try:
             return cls(value)
         except Exception as E:
-            choices = ', '.join([option.name for option in cls])
+            choices = ', '.join(normalize_to_display(m) for m in cls.__members__)
             raise ValueError(F'Could not transform {value} into {cls.__name__}; the choices are: {choices}') from E
 
     @classmethod
@@ -431,6 +455,40 @@ class Arg(Argument):
             metavar = 'B'
         return cls(*args, group=group, help=help, dest=dest, nargs=nargs, type=multibin, metavar=metavar)
 
+    @classmethod
+    def String(
+        cls,
+        *args   : str,
+        help    : Union[omit, str] = omit,
+        dest    : Union[omit, str] = omit,
+        nargs   : Union[omit, int, str] = omit,
+        metavar : Optional[str] = None,
+        group   : Optional[str] = None,
+    ):
+        """
+        Used to add argparse arguments that contain string data.
+        """
+        if metavar is None and any('-' in a for a in args):
+            metavar = 'STR'
+        return cls(*args, group=group, help=help, dest=dest, nargs=nargs, type=str, metavar=metavar)
+
+    @classmethod
+    def RegExp(
+        cls,
+        *args   : str,
+        help    : Union[omit, str] = omit,
+        dest    : Union[omit, str] = omit,
+        nargs   : Union[omit, int, str] = omit,
+        metavar : Optional[str] = None,
+        group   : Optional[str] = None,
+    ):
+        """
+        Used to add argparse arguments that contain a regular expression.
+        """
+        if metavar is None and any('-' in a for a in args):
+            metavar = 'REGEX'
+        return cls(*args, group=group, help=help, dest=dest, nargs=nargs, type=regexp, metavar=metavar)
+
     @classmethod
     def NumSeq(
         cls,
@@ -494,9 +552,9 @@ class Arg(Argument):
         """
         Used to add argparse arguments with a fixed set of options, based on an enumeration.
         """
-        cnames = [c.name.replace('_', '-') for c in choices]
+        cnames = [normalize_to_display(c).casefold() for c in choices.__members__]
         metavar = metavar or choices.__name__
-        return cls(*args, group=group, help=help, metavar=metavar, dest=dest, choices=cnames, type=str)
+        return cls(*args, group=group, help=help, metavar=metavar, dest=dest, choices=cnames, type=str.casefold)
 
     @classmethod
     def Choice(
@@ -533,7 +591,7 @@ class Arg(Argument):
         except KeyError:
             for a in self.args:
                 if a.startswith('--'):
-                    dest = a.lstrip('-').replace('-', '_')
+                    dest = normalize_to_identifier(a)
                     if dest.isidentifier():
                         return dest
             raise AttributeError(F'The argument with these values has no destination: {self!r}')
@@ -560,7 +618,7 @@ class Arg(Argument):
                 return sliceobj
             return annotation_type
 
-        name = pt.name.replace('_', '-')
+        name = normalize_to_display(pt.name, False)
         default = pt.default
         guessed_pos_args = []
         guessed_kwd_args = dict(dest=pt.name)
@@ -817,7 +875,7 @@ class Executable(ABCMeta):
                 if 'default' in known.kwargs:
                     known.kwargs.setdefault('nargs', OPTIONAL)
             elif not any(len(a) > 2 for a in known.args):
-                flagname = known.destination.replace('_', '-')
+                flagname = normalize_to_display(known.destination, False)
                 known.args.append(F'--{flagname}')
             action = known.kwargs.get('action', 'store')
             if action.startswith('store_'):
@@ -827,7 +885,7 @@ class Executable(ABCMeta):
                 known.kwargs.setdefault('type', multibin)
         return args
 
-    def __new__(mcs, name: str, bases: Sequence[Executable], nmspc: Dict[str, Any], abstract=False):
+    def __new__(mcs, name: str, bases: Sequence[Executable], nmspc: Dict[str, Any], abstract=False, extend_docs=False):
         def decorate(**decorations):
             for method, decorator in decorations.items():
                 try:
@@ -850,7 +908,7 @@ class Executable(ABCMeta):
         nmspc.setdefault('__doc__', '')
         return super(Executable, mcs).__new__(mcs, name, bases, nmspc)
 
-    def __init__(cls, name: str, bases: Sequence[Executable], nmspc: Dict[str, Any], abstract=False):
+    def __init__(cls, name: str, bases: Sequence[Executable], nmspc: Dict[str, Any], abstract=False, extend_docs=False):
         super(Executable, cls).__init__(name, bases, nmspc)
         cls._argument_specification = args = ArgumentSpecification()
 
@@ -866,6 +924,11 @@ class Executable(ABCMeta):
                 if key in parameters:
                     args[key] = value.__copy__()
 
+        if extend_docs and bases:
+            base_doc = bases[~0].__doc__.rstrip().lstrip('\n')
+            if base_doc:
+                cls.__doc__ = F'{cls.__doc__}\n{base_doc}'
+
         if not abstract and bases and has_keyword:
             for key, value in bases[0]._argument_specification.items():
                 if key not in args:
@@ -979,7 +1042,7 @@ class Executable(ABCMeta):
 
     @property
     def name(cls) -> str:
-        return cls.__name__.strip('_').replace('_', '-')
+        return normalize_to_display(cls.__name__)
 
     @property
     def logger(cls) -> Logger:
@@ -1262,7 +1325,6 @@ class Unit(UnitBase, abstract=True):
         return getattr(self.args, 'lenient', 0)
 
     def _exception_handler(self, exception: BaseException, data: Optional[ByteString]):
-        abort_execution = False
         if data is not None and self.leniency > 1:
             try:
                 return exception.partial
@@ -1282,7 +1344,6 @@ class Unit(UnitBase, abstract=True):
             raise exception
         elif isinstance(exception, VariableMissing):
             self.log_warn('critical error:', exception.args[0])
-            abort_execution = True
         elif isinstance(exception, GeneratorExit):
             raise exception
         elif isinstance(exception, RefineryImportMissing):
@@ -1309,8 +1370,6 @@ class Unit(UnitBase, abstract=True):
         if self.log_debug():
             import traceback
             traceback.print_exc(file=sys.stderr)
-        if abort_execution:
-            raise RefineryCriticalException(str(exception))
 
     def __next__(self) -> Chunk:
         if not self._chunks:
@@ -1752,12 +1811,9 @@ class Unit(UnitBase, abstract=True):
                 return pprint.pformat(message)
         message = ' '.join(transform(msg) for msg in messages)
         if clip:
-            from textwrap import shorten
             from refinery.lib.tools import get_terminal_size
-            message = shorten(
-                message,
-                get_terminal_size(75) - len(cls.name) - 14,
-            )
+            length = get_terminal_size(75) - len(cls.name) - 27
+            message = message[:length] + "..."
         return message
 
     @classmethod
