@@ -23,15 +23,20 @@ class FrameSlicer(Unit, abstract=True):
 
 
 class ConditionalUnit(Unit, abstract=True):
+    """
+    Conditional units can be used in two different ways. When a new frame opens after using this
+    unit, chunks that did not match the condition are moved out of scope for that frame but still
+    exist and will re-appear after the frame closes. When used inside a frame, however, the unit
+    works as a filter and will discard any chunks that do not match.
+    """
 
     def __init__(
         self,
         negate: Arg.Switch('-n', help='invert the logic of this filter; drop all matching chunks instead of keeping them') = False,
         single: Arg.Switch('-s', help='discard all chunks after filtering a single one that matches the condition') = False,
-        backup: Arg.Switch('-u', help='do not remove chunks from the frame entirely; move them out of scope instead') = False,
         **kwargs
     ):
-        super().__init__(negate=negate, single=single, backup=backup, **kwargs)
+        super().__init__(negate=negate, single=single, **kwargs)
 
     @abstractmethod
     def match(self, chunk) -> bool:
@@ -40,12 +45,12 @@ class ConditionalUnit(Unit, abstract=True):
     def filter(self, chunks: Iterable[Chunk]):
         single: bool = self.args.single
         negate: bool = self.args.negate
-        backup: bool = self.args.backup
+        nested: bool = self.args.nesting > 0
         for chunk in chunks:
             if chunk.visible and self.match(chunk) is negate:
-                if not backup:
+                if not nested:
                     continue
-                chunk.visible = False
+                chunk.set_next_scope(False)
             yield chunk
             if single:
                 break
