@@ -8,7 +8,7 @@ from refinery.lib.argformats import DelayedNumSeqArgument, PythonExpression
 from refinery.units.meta import Arg, ConditionalUnit
 
 
-class iff(ConditionalUnit):
+class iff(ConditionalUnit, extend_docs=True):
     """
     Filter incoming chunks depending on whether a given Python expression evaluates to true. If no
     expression is given, the unit filters out empty chunks.
@@ -18,23 +18,22 @@ class iff(ConditionalUnit):
         *expression: Arg(metavar='token', type=str, help=(
             'All "token" arguments to this unit are joined with spaces to produce the expression '
             'to be evaluated. This is done so that unnecessary shell quoting is avoided.')),
-        ge: Arg('-ge', type=str, metavar='<right-hand-side>', group='OP',
+        ge: Arg('-ge', type=str, metavar='RHS', group='OP',
             help='check that the expression is greater or equal to {varname}') = None,
-        gt: Arg('-gt', type=str, metavar='<right-hand-side>', group='OP',
+        gt: Arg('-gt', type=str, metavar='RHS', group='OP',
             help='check that the expression is greater than {varname}') = None,
-        le: Arg('-le', type=str, metavar='<right-hand-side>', group='OP',
+        le: Arg('-le', type=str, metavar='RHS', group='OP',
             help='check that the expression is less or equal to {varname}') = None,
-        lt: Arg('-lt', type=str, metavar='<right-hand-side>', group='OP',
+        lt: Arg('-lt', type=str, metavar='RHS', group='OP',
             help='check that the expression is less than {varname}') = None,
-        iN: Arg('-in', type=str, metavar='<right-hand-side>', group='OP',
-            help='check that the expression is contained in {varname}') = None,
-        ct: Arg('-ct', type=str, metavar='<right-hand-side>', group='OP',
+        ct: Arg('-ct', type=str, metavar='RHS', group='OP',
             help='check that the expression contains {varname}') = None,
-        eq: Arg('-eq', type=str, metavar='<right-hand-side>', group='OP',
+        iN: Arg('-in', '-i', type=str, metavar='RHS', group='OP',
+            help='check that the expression is contained in {varname}') = None,
+        eq: Arg('-eq', '-e', type=str, metavar='RHS', group='OP',
             help='check that the expression is equal to {varname}') = None,
         single=False,
         negate=False,
-        backup=False,
     ):
         operators = [
             (ge, operator.__ge__),
@@ -66,7 +65,6 @@ class iff(ConditionalUnit):
             rhs=rhs,
             cmp=cmp,
             negate=negate,
-            backup=backup,
             single=single,
         )
 
